#include <windows.h>
#include <process.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <time.h>
#include <Mmsystem.h>
#include <mmreg.h>
#include "Audio.h"
#include "LVSpeechPort.h"
#include <iostream>



bool bInit = TRUE;
bool bRecord;
bool bStop;
bool bRecording = FALSE;

unsigned long hThread;
unsigned int idThread;


int ibufrec;
int iBuffSize = 0;
int iRecTime;

WAVEFORMATEX	wfx;
HWAVEIN			hWaveIn;
WAVEHDR			whdr;

char szFileName[FILEPATH];
char audioBufIn[IN_BUFFER_SIZE];

SOUND_FORMAT	SoundFormat;	//Sent into LoadVoiceChannel to signify sound format

unsigned int __stdcall WaveMsgThread(void* vd);

///////////////////////////////////////////////////////////////
//Checks if the sound file format is one of the acceptable ones
//  ONLY:
//         WAVE_FORMAT_PCM	16 bit, 16kHz  PCM_16KHZ
//							 8 bit,  8kHz  PCM_8KHZ
//		   WAVE_FORMAT_ULAW  8 bit,  8kHz  ULAW_8KHZ
//
//  Returns true if MM_SOUND_FORMAT is one of the three above
//			false otherwise
//
SOUND_FORMAT GetSoundFormat()
{
	if (MM_SOUND_FORMAT == WAVE_FORMAT_PCM)
		{
		 if (SAMPLES_RATE_HZ == 16000 && SAMPLE_SIZE_BYTES == 2)
			{
			 SoundFormat = PCM_16KHZ;
			 return SoundFormat;
			}
		 if (SAMPLES_RATE_HZ == 8000 && SAMPLE_SIZE_BYTES == 2)
			{
			 SoundFormat = PCM_8KHZ;
			 return SoundFormat;
			}
		}

	if (MM_SOUND_FORMAT == WAVE_FORMAT_MULAW)
		 if (SAMPLES_RATE_HZ == 8000 && SAMPLE_SIZE_BYTES == 1)
			{
			 SoundFormat = ULAW_8KHZ;
			 return SoundFormat;
			}

	std::cout << "The sound format is not valid" << std::endl;
	return UNK_FORMAT;
}



///////////////////////////////////////////////////////////////
void InitSoundSystem()
{
	if(!bInit) return;
	
	
	memset(&wfx, 0, sizeof(WAVEFORMATEX));
	memset(&whdr, 0, sizeof(WAVEHDR));

	wfx.wFormatTag			= MM_SOUND_FORMAT;
    wfx.nChannels			= 1;
    wfx.nSamplesPerSec		= SAMPLES_RATE_HZ;
    wfx.nAvgBytesPerSec		= SAMPLES_RATE_HZ * SAMPLE_SIZE_BYTES;
    wfx.nBlockAlign			= SAMPLE_SIZE_BYTES;
    wfx.wBitsPerSample		= SAMPLE_SIZE_BYTES * 8;
    wfx.cbSize				= 0;

	hThread = _beginthreadex(NULL, 0, WaveMsgThread , NULL, 0, &(idThread));
	bInit = FALSE;
}

///////////////////////////////////////////////////////////////
int StartRecord()
{
	if (bInit)
	{
		InitSoundSystem();
	}

	iRecTime = 0;
	MMRESULT retVal;

	
	// Query the device to see if it supports the specified format:
	int Res = waveInOpen(&hWaveIn, WAVE_MAPPER, &wfx, 0, 0, WAVE_FORMAT_QUERY);

	if (Res == WAVERR_BADFORMAT) 
	{
		MessageBox(NULL, "Unhandled Audio Format Selected.","Audio",0);
		return -1;
	}



	if(( retVal = waveInOpen(&hWaveIn, WAVE_MAPPER, &wfx, MAKELONG(idThread, 0), NULL, CALLBACK_THREAD)) != MMSYSERR_NOERROR)
	{
		switch(retVal)
		{
		case MMSYSERR_ALLOCATED:
			MessageBox(NULL, "Resource already allocated.","Audio",0);
			break;

		case MMSYSERR_BADDEVICEID:
			MessageBox(NULL, "No such device present.","Audio",0);
			break;

		case MMSYSERR_NODRIVER:
			MessageBox(NULL, "No audio driver present.","Audio",0);
			break;

		case MMSYSERR_NOMEM:
			MessageBox(NULL, "Unable to allocate the memory need for the audio.","Audio",0);
			break;

		case WAVERR_BADFORMAT:
			MessageBox(NULL, "Audio device does not support the needed format.","Audio",0);
			break;
		}
		return -1;
	}
	ibufrec = 0;

	whdr.lpData				= audioBufIn;
	whdr.dwBufferLength		= IN_BUFFER_SIZE;
	whdr.dwBytesRecorded	= 0; 
	whdr.dwUser				= 0; 

	MMRESULT WaveResult;

	WaveResult = waveInPrepareHeader(hWaveIn, &whdr, sizeof(WAVEHDR));
	if(WaveResult != MMSYSERR_NOERROR)
		printf("Result for waveInPrepareHeader: %d\n", WaveResult);

	WaveResult = waveInAddBuffer(hWaveIn, &whdr, sizeof(WAVEHDR));
	if(WaveResult != MMSYSERR_NOERROR)
		printf("Result for waveInAddBuffer: %d\n", WaveResult);
	
	WaveResult = waveInStart(hWaveIn);
	if(WaveResult != MMSYSERR_NOERROR)
		printf("Result for waveInStart: %d\n", WaveResult);

	bRecording = TRUE;
	bStop = TRUE;

	printf("\n[Listening].");

	return 0;


}
///////////////////////////////////////////////////////////////
int StopRecord()
{
	if (!bStop) return 0;

	printf("[Stop]\n\n");
	if(bRecording)
	{
		MMRESULT WaveResult;

		Sleep(100);
		WaveResult = waveInStop(hWaveIn);
		if(WaveResult != MMSYSERR_NOERROR)
			printf("Result for waveInStop: %d\n", WaveResult);
		WaveResult = waveInReset(hWaveIn);
		if(WaveResult != MMSYSERR_NOERROR)
			printf("Result for waveInReset: %d\n", WaveResult);
		
		WaveResult = waveInUnprepareHeader(hWaveIn, &(whdr), sizeof(WAVEHDR));
		if(WaveResult != MMSYSERR_NOERROR)
			printf("Result for waveInUnprepareHeader: %d\n", WaveResult);

		WaveResult = waveInClose(hWaveIn);
		if(WaveResult != MMSYSERR_NOERROR)
			printf("Result for waveInClose: %d\n", WaveResult);

		bRecording = FALSE;
		bRecord = TRUE;
		bStop = FALSE;
	}
	return 0;
}

///////////////////////////////////////////////////////////////
// Returns pointer to audio buffer and also returns the current
// recording status (true = busy recording)
const char* GetAudio(int* iSize, bool *RecordingStatus)
{
	*iSize = IN_BUFFER_SIZE;

	*RecordingStatus = bRecording;

	return audioBufIn;
}
///////////////////////////////////////////////////////////////
unsigned int __stdcall WaveMsgThread(void* vd)
{
	MSG msg;
	WAVEHDR* wh;
	
	int retBuffer = ibufrec;
	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
		{
 
		 wh = (WAVEHDR* ) msg.lParam;
			 
		 switch(msg.message)
			{
			 case WIM_OPEN:
				retBuffer = 0;
				break;

			 case WIM_CLOSE:
				retBuffer = 1;
				break;

			 case WIM_DATA:
				StopRecord();
				break;
			}
		}
	return 0;
}
