#include "DemoRecognizer.h"
#include <iostream>
#include <string>

const std::string confirmgram = "builtin:grammar/boolean";

///////////////////////////////////////////////////////////////
clsDemoRecognizer::clsDemoRecognizer()
{
}

///////////////////////////////////////////////////////////////
clsDemoRecognizer::~clsDemoRecognizer()
{
	Port.Shutdown();
	Port.ClosePort();
}

///////////////////////////////////////////////////////////////
void clsDemoRecognizer::Log(const char *msg, void *userdata)
{
	std::cout << msg << std::endl;
}

///////////////////////////////////////////////////////////////
int clsDemoRecognizer::Initialize()
{
	int retv;

	if ((retv = Port.OpenPort(clsDemoRecognizer::Log, this)) != LV_SUCCESS)
	{
		std::cout << "Unable to open speech port. Error: " << retv << std::endl;
			return -1;
	}

	///////////////////////////////////////////////////////////////
	//We will use two voice channels.
	//Voice channel 1 is for regular decode, while voice channel 2 
	//is for confirmation decode. By doing thing, we will not wipe 
	//out previous decode result on voice channel 1 when we are 
	//doing confirmation recognition on voice channel 2.
	//
	//In this initiailzation function, we will load 
	//boolean grammar on voice channel 2, and keep it around.
	///////////////////////////////////////////////////////////////

	if ((retv = Port.LoadGrammar("confirmation", confirmgram.c_str())) != LV_SUCCESS)
	{
			std::cout << "Unable to load confirmation grammar. Error: " << retv << std::endl;
			return -1;
	}
	
	return 0;
}


///////////////////////////////////////////////////////////////
int clsDemoRecognizer::Recognize(const char *audio, unsigned int audio_length, SOUND_FORMAT format, const std::string & grammar_uri)
{

	///////////////////////////////////////////////
	//Deactivate grammars
	///////////////////////////////////////////////
	Port.DeactivateGrammars();

	int retv;

	retv = Port.LoadGrammar(grammar_uri.c_str());

	if (retv != LV_SUCCESS)
	{
		std::cout << "Unable to load grammar " << grammar_uri << ". Error: " << retv << std::endl;
		return -1;
	}
	///////////////////////////////////////////////
	//Activate a particular grammar
	///////////////////////////////////////////////
	retv = Port.ActivateGrammar(grammar_uri.c_str());
	if (retv != LV_SUCCESS)
	{
		std::cout << "Unable to activate grammar " << grammar_uri << ". Error: " << retv << std::endl;
		return -1;
	}

	///////////////////////////////////////////////
	//Load audio
	///////////////////////////////////////////////
	retv = Port.LoadVoiceChannel(1, (void*) audio, audio_length, format);
	if (retv != LV_SUCCESS)
	{
		std::cout << "Unable to load audio. Error: " << retv << std::endl;
		return -1;
	}

	///////////////////////////////////////////////
	//Set this property to 1, so every decode will be logged.
	///////////////////////////////////////////////
	Port.SetPropertyEx(PROP_EX_SAVE_SOUND_FILES, PROP_EX_VALUE_TYPE_INT,  (void*)1, PROP_EX_TARGET_CLIENT);

	///////////////////////////////////////////////
	//Do recognition.
	///////////////////////////////////////////////
	retv = Port.Decode(1, LV_ACTIVE_GRAMMAR_SET, LV_DECODE_SEMANTIC_INTERPRETATION);
	if (retv != LV_SUCCESS)
	{
		std::cout << "Unable to do decode  Error: " << retv << std::endl;
		return -1;
	}

	return 0;
}

///////////////////////////////////////////////////////////////
int clsDemoRecognizer::GetNumberOfInterpretations()
{
	return Port.GetNumberOfInterpretations(1);
}

///////////////////////////////////////////////////////////////
LVInterpretation clsDemoRecognizer::GetInterpretation(unsigned int index)
{
	return Port.GetInterpretation(1,index);
}

///////////////////////////////////////////////////////////////
int clsDemoRecognizer::Confirm(const char *audio, unsigned int audio_length, SOUND_FORMAT format)
{
	int retv;
	///////////////////////////////////////////////
	//Deactivate grammars
	///////////////////////////////////////////////
	Port.DeactivateGrammars();

	///////////////////////////////////////////////
	//Activate builtin boolean grammar.
	///////////////////////////////////////////////
	Port.ActivateGrammar("confirmation");


	///////////////////////////////////////////////
	//Load audio
	///////////////////////////////////////////////
	retv = Port.LoadVoiceChannel(2, (void*) audio, audio_length, format);
	if (retv != LV_SUCCESS)
	{
		std::cout << "Unable to load audio. Error: " << retv << std::endl;
		return -1;
	}

	///////////////////////////////////////////////
	//Set this property to 1, so every decode will be logged
	///////////////////////////////////////////////.
	Port.SetPropertyEx(PROP_EX_SAVE_SOUND_FILES, PROP_EX_VALUE_TYPE_INT,  (void*)1, PROP_EX_TARGET_CLIENT);

	///////////////////////////////////////////////
	//Do recognition.
	///////////////////////////////////////////////
	retv = Port.Decode(2, LV_ACTIVE_GRAMMAR_SET, LV_DECODE_SEMANTIC_INTERPRETATION);
	if (retv != LV_SUCCESS)
	{
		std::cout << "Unable to do decode  Error: " << retv << std::endl;
		return -1;
	}

	return 0;
}

///////////////////////////////////////////////////////////////
int clsDemoRecognizer::GetNumberOfConfirmInterpretation()
{
	return Port.GetNumberOfInterpretations(2);
}

///////////////////////////////////////////////////////////////
LVInterpretation clsDemoRecognizer::GetConfirmInterpretation()
{
	return Port.GetInterpretation(2,0);
}

